/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/

#include <barsread.h>
#include <S60Bible.rsg>
#include "S60Bible.hrh"
#include "BibleSettingList.h"

CBibleSettingList *CBibleSettingList::NewL(CAknView *aView)
{
	CBibleSettingList* self = CBibleSettingList::NewLC(aView);
	CleanupStack::Pop(self);
	return self;
}

CBibleSettingList *CBibleSettingList::NewLC(CAknView *aView)
{
	CBibleSettingList* self = new (ELeave) CBibleSettingList(aView);
	CleanupStack::PushL(self);
	return self;
}

CBibleSettingList::CBibleSettingList(CAknView *aView) :
	CAknSettingItemList()
{
	iView = aView;
	SetRect(Rect());
	SizeChanged();
}

CBibleSettingList::~CBibleSettingList()
{
	// no specific destruction code required - no owned data
}


void CBibleSettingList::SizeChanged()
{
	// if size changes, make sure component takes whole available space
	CEikFormattedCellListBox *listbox = ListBox();

	if (listbox) {
		listbox->SetRect(Rect());
	}
}

void CBibleSettingList::EditCurrentItemL()
{
	EditItemL(ListBox()->CurrentItemIndex(), ETrue); // invoked from menu
}


void CBibleSettingList::Draw(const TRect& /*aRect*/) const
{
}

CAknSettingItem * CBibleSettingList::CreateSettingItemL (TInt aIdentifier)
{
	CAknSettingItem* settingItem = NULL;
    	switch (aIdentifier) {
		case ESettingBackLight:
			{
				settingItem = new (ELeave) CAknBinaryPopupSettingItem (
					aIdentifier,
					iBackLight);
				break;
			}
		case ESettingFullScreen:
			{
				settingItem = new (ELeave) CAknBinaryPopupSettingItem (
					aIdentifier,
					iFullScreen);
				break;
			}
		case ESettingFullBookName:
			{
				settingItem = new (ELeave) CAknBinaryPopupSettingItem (
					aIdentifier,
					iFullBookName);
				break;
			}

		default:
		break;
	}
    	return settingItem;
}
