/*
 *  ColorChooser.cpp
 *  
 *
 *  Created by Yohanes Nugroho on 05/04/06.
 *  Copyright 2006 __MyCompanyName__. All rights reserved.
 *
 */

#include "ColorChooser.h"
#include <coemain.h>                                                            
#include <barsread.h>                                                           
#include <eiksbfrm.h>
#include <eikenv.h>
#include <eikappui.h>
#include <gdi.h>
#include "log.h"

#define COLOR_ROW 3
#define COLOR_COL 10
#define MAX_COLOR (COLOR_ROW*COLOR_COL)

const int MAX_ACCELERATION = 20;
const TUint DIFF_TIME = 500000;

const TUint32 colors[MAX_COLOR] = 
{
	0xFFFFFF, 0xC0C0C0, 0x808080,
	0x404040, 0x0000FF, 0xAFAFFF,
	0xFFC800, 0x00FFFF, 0x00FF00,
	0xFF00FF, 0xFFFF00, 0xFF0000,
	0xF1F1F1, 0xC0C0C0, 0x808080,
	0x404040, 0x0000FF, 0xAFAFFF,
	0xFFC800, 0x00FFFF, 0x00FF00,
	0xFF00FF, 0xFFFF00, 0xFF0000,
	0xFFC800, 0x00FFFF, 0x00FF00,
	0xFF00FF, 0xFFFF00, 0xFF0000,	
};

CColorChooser::CColorChooser(TRgb aRGB): iAcceleration(1)
{
//	Log(_L("Constructor called"));
	iCurrentRGB = aRGB.Value();
} 

CColorChooser::~CColorChooser()
{
//	Log(_L("Destructor called"));
	delete iRedLabel;
	delete iGreenLabel;
	delete iBlueLabel;
}

void  CColorChooser::SizeChanged()
{
			//	Log(_L("Size Changed"));
}

TSize CColorChooser::MinimumSize()
{
	return TSize(160, 150);
}

TRgb CColorChooser::GetCurrentColor()
{
	return TRgb(iCurrentRGB);
}


void CColorChooser::DrawScroll(int linewidth, 
							   int linecenter, 
							   int vline, 
							   int value, 
							   HBufC* str,
							   TBool selected) const
{
    CGraphicsContext& gc=SystemGc();
	
	
	if (selected) {
		gc.SetBrushColor(KRgbWhite);
		gc.SetPenColor(KRgbBlue);
		TRect sel = TRect(TPoint(linecenter-3, vline-3), TSize(linewidth+6,8));	
		gc.DrawRect(sel);
	}
	gc.SetBrushColor(KRgbBlack);
	gc.SetPenColor(KRgbBlack);
	
	
	gc.DrawLine(TPoint(linecenter, vline), TPoint(linecenter+linewidth, vline));
	TRect pos = TRect(TPoint(linecenter + (linewidth*value/255), vline-2), TSize(4,6));
	
	
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	if (selected) {
		gc.SetBrushColor(KRgbRed);
	} else {
		gc.SetBrushColor(KRgbBlack);
	}
	gc.SetPenColor(KRgbBlack);
	gc.DrawRect(pos);
	TBuf<3> num;
	gc.SetBrushColor(KRgbGreen);
	gc.SetPenColor(KRgbBlue);
	gc.SetBrushStyle(CGraphicsContext::ENullBrush);	
	
	num.SetLength(0);
	num.AppendNum(value);
	int l = linecenter*2/3; //space on right of scrollbar
	const CFont *font = CEikonEnv::Static()->DenseFont();
	int h = font->HeightInPixels();
	TRect numbox(TPoint(linecenter+linewidth+5, vline-h/2), TSize(l, h)); 
	gc.DrawText(num, numbox, font->AscentInPixels(),CGraphicsContext::ECenter);
	TRect labelBox(TPoint(3, vline-h/2), TSize(linecenter, h)); 
	gc.DrawText(*str, labelBox, font->AscentInPixels(),CGraphicsContext::ECenter);
	
}


void CColorChooser::Draw(const TRect& aRect) const
{
    // draw surround                                                            
    CGraphicsContext& gc=SystemGc(); // context to draw into                    
    gc.UseFont(CEikonEnv::Static()->DenseFont());
	
																																							                                                                            
    TRect rect=Rect(); // screen boundary    
	gc.DrawRect(rect); // outline screen boundary                               
    rect.Shrink(1,1);                                                           
    gc.SetPenColor(KRgbWhite);                                                  
    gc.DrawRect(rect);                                                          
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	int previewwidth = rect.Width()*2/3;
	int previewheight = rect.Height()/7;
	int center = (rect.Width()-previewwidth)/2;
	TRect preview = TRect(TPoint(center, 5), TSize(previewwidth, previewheight));
	gc.SetBrushColor(iCurrentRGB);
	gc.DrawRect(preview);
	gc.SetBrushStyle(CGraphicsContext::ENullBrush);
	gc.SetPenColor(KRgbBlack);
	preview.Grow(1,1);
	gc.DrawRect(preview);

	int cdisplayheight = (rect.Height()/3);
	int w = (rect.Width()-5)/COLOR_COL;
	int h = cdisplayheight/COLOR_ROW;
	TRect r = TRect(TPoint(0, rect.Height()-cdisplayheight), TSize(w, h));
	r.Shrink(2,2);
	
	//space between preview and color box
	int spacebetween = rect.Height()-cdisplayheight-previewheight;
	int linespace = spacebetween/4;
	int linewidth = rect.Width()/2;
	int linecenter = (rect.Width()-rect.Width()/2)/2;
	int vline = previewheight+linespace;
	
	TRgb rgb(iCurrentRGB);
	DrawScroll(linewidth, linecenter, vline, rgb.Red(), iRedLabel, iSelected==ERedSelected);	
	
	vline+=linespace;
	DrawScroll(linewidth, linecenter, vline, rgb.Green(), iGreenLabel, iSelected==EGreenSelected);
	vline+=linespace;
	DrawScroll(linewidth, linecenter, vline, rgb.Blue(), iBlueLabel, iSelected==EBlueSelected);
	
	for (int i = 0; i<COLOR_ROW; i++){
		TRect rtmp = r;
		rtmp.Move(5, h*i);
		for (int j = 0; j<COLOR_COL; j++){
			if (iSelected>=EColorSelected && (i*COLOR_COL+j==iSelected-EColorSelected)){
				TRect tframe = rtmp;
				gc.SetBrushColor(KRgbWhite);
				gc.SetPenColor(KRgbBlack);
				tframe.Grow(2,2);
				gc.DrawRect(tframe);
				gc.SetPenColor(KRgbWhite);
			}
			gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
			gc.SetBrushColor(colors[i*COLOR_COL+j]);
			gc.DrawRect(rtmp);
			gc.SetBrushStyle(CGraphicsContext::ENullBrush);
			gc.SetPenColor(KRgbGray);
			rtmp.Grow(1,1);
			gc.DrawRect(rtmp);
			rtmp.Shrink(1,1);
			rtmp.Move(w, 0);
		}
	}
	gc.DiscardFont();
}

TCoeInputCapabilities CColorChooser::InputCapabilities() const
{
	//return TCoeInputCapabilities::ENone;
	return TCoeInputCapabilities::ENavigation;
}

TKeyResponse CColorChooser::OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
{
	TRgb rgb(iCurrentRGB);
	TTime current;
	current.HomeTime();
	if (iSelected<EColorSelected){
		if (aType ==  EEventKey)                                                    
		{	
			
			if (aKeyEvent.iScanCode==aLastKey && 
				(current.MicroSecondsFrom(iTime).Int64()<DIFF_TIME)) {
				iAcceleration = iAcceleration>=MAX_ACCELERATION
					?iAcceleration:iAcceleration*2;
			} else {
				iAcceleration = 1;
			}
			switch (aKeyEvent.iScanCode)                                            
			{
				case EStdKeyLeftArrow:                                                  
				{     
					aLastKey = EStdKeyLeftArrow;
					switch (iSelected) {
						case ERedSelected:
							if (rgb.Red()>iAcceleration){
								rgb.SetRed(rgb.Red()-iAcceleration);
							} else {
								rgb.SetRed(0);
							}
							break;
						case EGreenSelected:
							if (rgb.Green()>iAcceleration){
								rgb.SetGreen(rgb.Green()-iAcceleration);
							} else {
								rgb.SetGreen(0);
							}
							break;							
						case EBlueSelected:
							if (rgb.Blue()>iAcceleration){
								rgb.SetBlue(rgb.Blue()-iAcceleration);
							} else {
								rgb.SetBlue(0);
							}
							break;
					}
					break;                                                          
				}	
					
				case EStdKeyRightArrow:                                                  
				{
					aLastKey = EStdKeyRightArrow;
					switch (iSelected) {
						case ERedSelected:
							if (rgb.Red()<255-iAcceleration){
								rgb.SetRed(rgb.Red()+iAcceleration);
							} else {
								rgb.SetRed(255);
							}
							break;
						case EGreenSelected:
							if (rgb.Green()<255-iAcceleration){
								rgb.SetGreen(rgb.Green()+iAcceleration);
							} else {
								rgb.SetGreen(255);
							}
							break;							
						case EBlueSelected:
							if (rgb.Blue()<255-iAcceleration){
								rgb.SetBlue(rgb.Blue()+iAcceleration);
							} else {
								rgb.SetBlue(255);
							}
							break;
					}					
					break;                                                          
				}	
					
					
				case EStdKeyDownArrow:                                                  
				{     
					iSelected++;
					break;                                                          
				}	
				case EStdKeyUpArrow:                                                    
				{                                                                   
					iSelected--;
					break;                                                          
				}
			}
			
		}
		iCurrentRGB = rgb.Value();
		iTime.HomeTime();
	}  else if (aType ==  EEventKey)                                                    
    {                                                                           
        switch (aKeyEvent.iScanCode)                                            
        {                                                                       
			case EStdKeyLeftArrow:                                                  
            {     
				iSelected--;
				break;                                                          
            }	

			case EStdKeyRightArrow:                                                  
            {     
				iSelected++;
				break;                                                          
            }	

			case EStdKeyDownArrow:                                                  
            {     
				iSelected+=COLOR_COL;
				break;                                                          
            }	
			case EStdKeyUpArrow:                                                    
            {                                                                   
				if (iSelected<COLOR_COL) {
					iSelected=EBlueSelected;
				} else {
					iSelected-=COLOR_COL;
				}
				break;                                                          
            }                                                                   
        }
	}
		if (iSelected>=MAX_COLOR+EColorSelected) iSelected=ERedSelected;

		if (iSelected<0) iSelected = EColorSelected+MAX_COLOR-1;
		DrawDeferred();
	
	return CCoeControl::OfferKeyEventL(aKeyEvent, aType);
}
	
void CColorChooser::ConstructFromResourceL(TResourceReader& aReader)
{
//	Log(_L("Constructing From Resource"));
	
    TInt width=aReader.ReadInt16();                                             
    TInt height=aReader.ReadInt16();                      
		TSize containerSize;
#ifdef __SERIES60_30__	
	TRect apprect = CEikonEnv::Static()->EikAppUi()->ApplicationRect();
	if (apprect.Height()>210 && apprect.Width()>210) {
		containerSize.SetSize (apprect.Width()*4/5, apprect.Height()*4/5); 
	}
#else
    containerSize.SetSize  (width, height); 
#endif
                                                                                                                       
    SetSize(containerSize);
	
	iRedLabel = aReader.ReadTPtrC16().AllocL();
	iGreenLabel = aReader.ReadTPtrC16().AllocL();
	iBlueLabel= aReader.ReadTPtrC16().AllocL();
	iSelected = ERedSelected;
	ActivateL();
}


void CColorChooser::DoSetRGB()
{
		if (iSelected>=EColorSelected) {
			iCurrentRGB = colors[iSelected-EColorSelected];
			DrawDeferred();
		}
}
