/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


#ifndef __BOOKLISTMAINVIEW_H__
#define __BOOKLISTMAINVIEW_H__

#include <aknappui.h>
#include <coecntrl.h>
#include <e32std.h>
#include <aknappui.h>
#include <eikmenup.h>
#include <AknWaitDialog.h>
#include <aknview.h>
#include <aknquerydialog.h>

class	CS60BibleDocument;
class CBookListContainer;

/*!
  @class CS60BibleAppView
  @discussion An instance of class CS60BibleAppView is the Application View object for the
  GUI / Engine example
  */
class CBookListMainView :  public CAknView, public MProgressDialogCallback
    {
public:
	
	
    TUid Id() const;
    CBookListMainView();
    void ConstructL();

    void  DialogDismissedL (  TInt    aButtonId  );

/*!
  @function NewL
  @abstract Creates a CS60BibleAppView object, which will draw itself to aRect
  @param aRect A rectangle that defines the size and location of the displayable area
  for the view
  @param aDocument the document
  */
    static CBookListMainView * NewL(const TRect& aRect, CS60BibleDocument* aDocument);

/*!
  @function NewLC
  @abstract Creates a CS60BibleMainView object, which will draw itself to aRect
  @param aRect A rectangle that defines the size and location of the displayable area
  for the view
  @param aDocument the document
  */
    static CBookListMainView * NewLC(const TRect& aRect, CS60BibleDocument* aDocument);

/*!
  @function	~CS60BibleMainView
  @abstract Destroys the object
  */
    ~CBookListMainView ();


public:



    void DoActivateL(const TVwsViewId& /*aPrevViewId*/,
                                    TUid /*aCustomMessageId*/,
                                    const TDesC8& /*aCustomMessage*/);
    void DoDeactivate();

    void HandleCommandL(TInt aCommand);

    void SwitchBack();
      void SetRetVal(int i);

private: // New methods

/*!
  @function CS60BibleMainView
  @abstract Performs the first stage construction
  @param aModel the model
  */
    CBookListMainView(CS60BibleDocument* aDocument);

/*!
  @function ConstructL
  @abstract Performs the second phase construction, setting the bounding
  rectangle to aRect
  @param aRect the display area for the view
  */
    void ConstructL(const TRect& aRect);
    


/*!
  @function HandlePointerEventL
  @abstract Handles any pointer events
  @param aPointerEvent The event

    void HandlePointerEventL(const TPointerEvent& aPointerEvent);
  */
  


private:

/*! @var iDocument The document	*/
	CS60BibleDocument* iDocument;

    	CBookListContainer* iContainer;
    	TVwsViewId iPrevView;
    	TUid iRetVal;
    	
};


#endif // __S60BibleAPPVIEW_H__
