/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


#ifndef __MAIN_CONTAINER_H__
#define __MAIN_CONTAINER_H__

#include <aknview.h>
#include <aknutils.h> 
#include "S60BibleDocument.h"

class CS60BibleMainContainer : public CCoeControl
    {
public:

/*!
  @function NewL

  @discussion Create a CMultiViewsContainer1 object, which will draw itself to aRect
  @param aRect the rectangle this view will be drawn to
  @result a pointer to the created instance of CMultiViewsContainer1
*/
    static CS60BibleMainContainer* NewL(CAknView *aView, CS60BibleDocument* aDocument);

/*!
  @function NewLC

  @discussion Create a CMultiViewsContainer1 object, which will draw itself to aRect
  @param aRect the rectangle this view will be drawn to
  @result a pointer to the created instance of CMultiViewsContainer1
*/
    static CS60BibleMainContainer* NewLC(CAknView *aView, CS60BibleDocument* aDocument);

/*!
  @fuction ConstructL

  @discussion Perform the second phase construction of a CMultiViewsContainer1 object
  @param aRect Frame rectangle for container.
*/
    void ConstructL(CAknView *aView, CS60BibleDocument* aDocument);

public: // from CoeControl
/*!
  @function CountComponentControls

  @discussion Get the number of controls contained in this compound control
  @return Number of component controls
*/
    TInt CountComponentControls() const;

/*!
  @function ComponentControl.

  @discussion Get the specified component
  @param aIndex specification for component pointer
  @return Pointer to component control
*/
    CCoeControl* ComponentControl(TInt aIndex) const;

/*!
  @function Draw

  @discussion Draw this CMultiViewsContainer1 to the screen
  @param aRect the rectangle of this view that needs updating
*/
    void Draw(const TRect& aRect) const;

#ifdef __SERIES60_30__
	void HandleResourceChange(TInt aType);
#endif

	CS60BibleDocument* iDocument;

    TKeyResponse OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType);
    TCoeInputCapabilities InputCapabilities() const;
    CEikButtonGroupContainer* iCba;
    TBool iFullScreen;
    void SetFullScreenL(TBool aFullScreen);
    void ToggleFullScreen();
    CAknView *iView;
	void SetBackLight(TBool aAlwaysOn);
	~CS60BibleMainContainer();
	void ExecuteCommand(int idx);
private:
	static TInt Tick(TAny* aObject);
	CPeriodic* iTimer;
    };

#endif
