/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    Original work (PalmBible and Bible+):
    Copyright (C) 2001  Ho Chuen Poon
    Modifications (C) 2003 Yih-Chun Hu

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/



#ifndef VERSIONINFO_H
#define VERSIONINFO_H

#include "const.h"
#include "info.h"
#include "Database.h"
#include "util.h"

class VersionInfo;

class VersionInfo {
	public:
		IMPORT_C VersionInfo(Database *dbase);
		IMPORT_C ~VersionInfo();
		char GetSepChar();

		bool GetCopyProtected(UInt16 version);
		bool GetByteShifted(UInt16 version);
		bool GetRightAligned(UInt16 version);

		char* GetVersionName(UInt16 version);
		char* GetVersionInfo(UInt16 version);

		char* GetSimpleBookName(UInt16 version, UInt16 book);
		ListContainPtr GetSimpleBookList(UInt16 version);

		char* GetComplexBookName(UInt16 version, UInt16 book);

		UInt16 GetTotalVersion();
		UInt16 GetTotalBook(UInt16 version);
		UInt16 GetBookRecordIndex(UInt16 version, UInt16 book);
		UInt16 GetBookTotalRecord(UInt16 version, UInt16 book);
		UInt16 GetWordRecordIndex(UInt16 version);
		UInt16 GetWordTotalRecord(UInt16 version);
		UInt16 GetTotalChapter(UInt16 version, UInt16 book);
		UInt16 GetTotalVerse(UInt16 version, UInt16 book, UInt16 chapter);
		int GetBookIndex(char *bookname);
		Database *getDatabase();
		int GetNextBook(int currentbook);
		int GetPrevBook(int currentbook);
		int GetFirstBook();
	private:

		Database *dbase;
		void lockVersionPtr();
		void unlockVersionPtr();
		UInt16 infVersion;
		BookInfoPtr lockBookPtr();
		void unlockBookPtr();
		char* getVersionInfo(bool name);
		UInt16 getWordRecord();
		UInt16 getWordTotal();
		UInt16 getBookRecord();
		UInt16 getBookTotal();
		char* getBookName(bool complex);
		UInt16 getTotalBook();

		VersionInfoPtr infVersionInfoPtr;
		UInt16 infBook;
		UInt16 infChapter;
		UInt16 infVerse;
};


#endif
