/*
    S60Bible A Bible Reader for SymbianOS
    Copyright (C) 2004 Yohanes Nugroho

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Yohanes Nugroho (yohanes@opensource.or.id)
    Kp Areman RT 09/08 No 71 Desa Tugu Cimanggis Depok 16951 Indonesia
*/


#ifndef __VIEWER_H
#define __VIEWER_H
#include <e32std.h>
#include <f32file.h>
#include <eikdoc.h>
#include <s32file.h>
#include <txtglobl.h>
#include <frmtview.h>
#include <txtfmlyr.h>
#include <badesca.h>
#include <gdi.h>
#include <charconv.h>
#include "const.h"

class Reader;
class VerseInfo;
class CStringList;

#define MAX_ZOOM 5


typedef struct {
	int number;
	char *name;
} bookinfo_t;

const bookinfo_t englishbooknames[] = {
{10, "Genesis"},
{20, "Exodus"},
{30, "Leviticus"},
{40, "Numbers"},
{50, "Deuteronomy"},
{60, "Joshua"},
{70, "Judges"},
{80, "Ruth"},
{90, "1 Samuel"},
{100, "2 Samuel"},
{110, "1 Kings"},
{120, "2 Kings"},
{130, "1 Chronicles"},
{140, "2 Chronicles"},
{150, "Ezra"},
{160, "Nehemiah"},
/*Catholic only*/
{170, "Tobit"},
{175, "Tobias"},
{180, "Judit"},
/*End of Catholic only*/
{190, "Esther"},
/*Catholic only*/
{200, "1 Maccabees"},
{210, "2 Maccabees"},
/*End of Catholic only*/
{220, "Job"},
{230, "Psalms"},
{240, "Proverbs"},
{250, "Ecclesiastes"},
{260, "Song of Solomon"},
/*Catholic only*/
{270, "Wisdom"},
{280, "Sirach"},
/*End of Catholic only*/
{290, "Isaiah"},
{300, "Jeremiah"},
{310, "Lamentations"},
/*Catholic only*/
{320, "Baruch"},
/*End of Catholic only*/
{330, "Ezekiel"},
{340, "Daniel"},
{350, "Hosea"},
{360, "Joel"},
{370, "Amos"},
{380, "Obadiah"},
{390, "Jonah"},
{400, "Micah"},
{410, "Nahum"},
{420, "Habakkuk"},
{430, "Zephaniah"},
{440, "Haggai"},
{450, "Zechariah"},
{460, "Malachi"},
{470, "Matthew"},
{480, "Mark"},
{490, "Luke"},
{500, "John"},
{510, "Acts"},
{520, "Romans"},
{530, "1 Corinthians"},
{540, "2 Corinthians"},
{550, "Galatians"},
{560, "Ephesians"},
{570, "Philippians"},
{580, "Colossians"},
{590, "1 Thessalonians"},
{600, "2 Thessalonians"},
{610, "1 Timothy"},
{620, "2 Timothy"},
{630, "Titus"},
{640, "Philemon"},
{650, "Hebrews"},
{660, "James"},
{670, "1 Peter"},
{680, "2 Peter"},
{690, "1 John"},
{700, "2 John"},
{710, "3 John"},
{720, "Jude"},
{730, "Revelation"},
/*Apocrypha*/
{740, "1 Esdras"},
{750, "2 Esdras"}
};

const int booknames_size = sizeof(englishbooknames)/sizeof(bookinfo_t);

class Viewer :public CBase {
public:
	Viewer();
	~Viewer();
	void Init();
	void ConstructL(RFs &aFs);
	void ScrollUp(int count);
	void ScrollDown(int count);
	void UpdateBuffer();
	TBool ChangeBibleL(const TDesC &filename);
	TBool ChangeBible(const TDesC &filename);
	int GetStartChapter();
	int GetStartVerse();
	int GetStartLine();
	char * BookName();
	char* SimpleBookName();

	void ClearVerse();
	void ClearVerseList();
	void FillVerse();
	VerseInfo* GetVerseHead();
	bool SeekVerse(UInt16 version, UInt16 book, UInt16 chapter, UInt16 verse);
	bool SeekVerseAsString(UInt16 version, char *str);
	void PageDown();
	void PageUp();
	int IsReady();
	void NextVerse();
	void PrevVerse();
	void GotoTop();
	void GotoBottom();
	void SaveState();
	void LoadState();

    void SetArea(const TRect &aRect);
    void NextChapter();
    void PrevChapter();
    void NextBook();
    void PrevBook();

    int GetBookNumber();

    void CopyToClipboardL(int count);
    const CFont *GetFont();
	void setFont(int num);
    CDesCArray* GetBookList();
	void  ClearBookList();
	void GotoBook(int book);
	void GotoBookByIndex(int idx);
	void LoadBookmarkL();
	void SaveBookmarkL();
	CDesCArray* GetBookmark();
	void AddToBookmark();
	void DelBookmark(int idx);
	void DelAllBookmark();

	TInt& GetBinding(int idx);

	void LoadBindings();
	void SaveBindings();


	TFileName iCurrentBook;
	TFileName iBindingsFile;
	TFileName iOptionsFile;
	TFileName iBookmarkFile;

	void SetEncoding(TUint Encoding);
	void StringToDes(const char *aStr, TDes &aTarget);
	void DesToString(const TDesC &aDes, char *aTarget);
	void MyDesToString(const TDesC &aDes, char *aTarget);
	void MyStringToDes(const char *aStr, TDes &aTarget);
	void HebrewStringToDes(const char *aStr, TDes &aTarget);
	TUint GetEncoding();
	int GetFontNum();
	int VerseExisted(int chapter, int verse);
	void Viewer::Add(VerseInfo *v);
	TBool IsRToL();
	void SetLongBookName(TBool aLongname);
	TBool UseLongBookName();
	
	enum ETextObject {
		EVerseText,
		EVerseBackground,
		EVerseNumber,
		EHeaderText,
		EHeaderFrame,
		EHeaderBG,
	};
	void SetColor(ETextObject aTextObject, TRgb aColor);
	TRgb GetColor(ETextObject aTextObject) const;
	void SetZoomLevel(int level);
	int GetZoomLevel();
	void ZoomIn();
	void ZoomOut();
	void WrapVerse(VerseInfo *v);
	int GetTotalBookChapter(int book);
	int GetTotalChapterVerse(int book, int chapter);
	int GetBookIndex();
	
	int GetGlobal(int index);
	void SetGlobal(int index, int value);

	enum {
		EGlobalFindFocus,	
	};

private:
	int iZoomLevel;
	int zoom[MAX_ZOOM];
	void CalcScrollLen();

	CEikDocument *iDoc;
    TRgb iColors[6];

	void AddLast(VerseInfo *v);
	void AddFirst(VerseInfo *v);
	void DelFirst();
	void DelLast();
	void FillBookList();
	void Big5StringToDes(const char *aStr, TDes &aTarget);
	void GreekStringToDes(const char *aStr, TDes &aTarget);
	int IsScreenCovered();
	int AddVerse();
	void TrimDown();
	void TrimUp();

	VerseInfo* iVerseHead;
	VerseInfo* iVerseTail;
	
	int iStartChapter, iStartVerse, iStartLine;
	int iScrollLen;
	RFs iFs;
    TRect iArea;
    const CFont *iFont;

	TInt iNormalWidth;
	TInt iFirstVerseWidth;

	CFont *iCustomFont;
    CDesCArray* iBookList;
	CStringList *iBookmarkList;
	
	Reader *iReader;
	TBool isHebrew;	
	int iBookNumberList[100];
	TInt iBinding[20];
	/*80 kb*/
	TBuf<16384> iVerseContent;
	char iTmpBuff[16384];
	UInt16 iLinePos[8192];
	UInt16 iLineFormat[8192];
	int iLineCount;
	
	TUint iCurrentEncoding;
	int iFontNum;
	int iLastFont;
	int iLastEnc;
	TBool iLongName;
	
	int tmpStartVerse,tmpStartChapter, tmpStartLine;	

	TBool isGreek;
	TBool isBIG5;
	CCnvCharacterSetConverter::TAvailability iConverterAvailability;
	CCnvCharacterSetConverter* iCharacterConverter;
	TInt iGlobal[10];
	
};

#endif
